package com.brilliance.swift.mt2mx.camt054001;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.ERROR;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.camt029001.impl.*;
import com.brilliance.swift.mt2mx.camt054001.impl.*;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.SwiftTransferUtil;
import com.brilliance.swift.vo.SwiftTranslationErrorInfo;
import com.brilliance.swift.vo.SwiftTranslationReport;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field32A;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxCamt054001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "camt.054.001.08");
        Map<String, Object> appHdrMaps = (Map<String, Object>)jsonMaps.get("appHdr");
        appHdrMaps.put("msgDefIdr", "camt.054.001.08");
        AbstractMT abstractMT = context.get(AbstractMT.class);

        //初始化转换和不需要BLOCK4的转换
        Map<String, Object> bkToCstmrDbtCdtNtfctnMaps = new HashMap<>();
        jsonMaps.put("bkToCstmrDbtCdtNtfctn", bkToCstmrDbtCdtNtfctnMaps);
        Map<String, Object> grpHdrMaps = new HashMap<>();
        bkToCstmrDbtCdtNtfctnMaps.put("grpHdr", grpHdrMaps);
        grpHdrMaps.put("creDtTm", getCreateDateStr());
        grpHdrMaps.put("nbOfTxs", "1");

        Map<String, Object> ntfctnMaps = new HashMap<>();
        JSONArray ntfctnJsonArray = new JSONArray();
        ntfctnJsonArray.add(ntfctnMaps);
        bkToCstmrDbtCdtNtfctnMaps.put("ntfctn", ntfctnJsonArray);
        Map<String, Object> ntryMaps = new HashMap<>();
        JSONArray ntryJsonArray = new JSONArray();
        ntryJsonArray.add(ntryMaps);
        ntfctnMaps.put("ntry",ntryJsonArray);
        Map<String, Object> ntryDtlsMaps = new HashMap<>();
        JSONArray ntryDtlsJsonArray = new JSONArray();
        ntryDtlsJsonArray.add(ntryDtlsMaps);
        ntryMaps.put("ntryDtls",ntryDtlsJsonArray);
        Map<String, Object> txDtlsMaps = new HashMap<>();
        JSONArray txDtlsJsonArray = new JSONArray();
        txDtlsJsonArray.add(txDtlsMaps);
        ntryDtlsMaps.put("txDtls",txDtlsJsonArray);
        Map<String, Object> stsMaps = new HashMap<>();
        ntryMaps.put("sts",stsMaps);
        Map<String, Object> bkTxCdMaps = new HashMap<>();
        stsMaps.put("cd","BOOK");
        ntryMaps.put("bkTxCd",bkTxCdMaps);
        Map<String, Object> prtryMaps = new HashMap<>();
        bkTxCdMaps.put("prtry",prtryMaps);
        prtryMaps.put("cd",Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        prtryMaps.put("issr",Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        ntryMaps.put("ntryRef",Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        if("900".equals(abstractMT.getMessageType())){
            ntryMaps.put("cdtDbtInd", CreditDebitCode.DBIT);
            txDtlsMaps.put("cdtDbtInd",CreditDebitCode.DBIT);
        } else if("910".equals(abstractMT.getMessageType())){
            ntryMaps.put("cdtDbtInd", CreditDebitCode.CRDT);
            txDtlsMaps.put("cdtDbtInd",CreditDebitCode.CRDT);
        }
        Tag tag32A = abstractMT.getSwiftMessage().getBlock4().getTagByName("32A");
        if(tag32A!=null){
            Field32A field32A = (Field32A)tag32A.asField();
            if(field32A.getValue().length()>10){
                Map<String, Object> acctMaps = null;
                if(ntfctnMaps.containsKey("acct")){
                    acctMaps = (Map<String, Object>)ntfctnMaps.get("acct");
                }else {
                    acctMaps = new HashMap<>();
                    ntfctnMaps.put("acct",acctMaps);
                }
                buildSTErrorInfo(ERROR.T20200, "Ntfctn/Acct/Ccy", null);
                acctMaps.put("ccy",field32A.getValue().substring(6,9));
            }
        }

        String uetr = abstractMT.getSwiftMessage().getUETR();
        if (StringUtil.isNotEmpty(uetr)) {
            Map<String, Object> refsMaps = new HashMap<>();
            txDtlsMaps.put("refs",refsMaps);
            refsMaps.put("uetr",uetr);
        }
        context.set(Mt2MxContextIdentifier.MT_TO_MX_FLAG_50,"false");
    }

    protected void buildSTErrorInfo(String errorCode, String location, String originalValue) {
        SwiftTranslationReport str = context.get(SwiftTranslationReport.class);
        SwiftTranslationErrorInfo errorInfo = SwiftTransferUtil.getSwiftTranslationErrorInfoByErrorCode(errorCode);
        errorInfo.setLocation(location);
        errorInfo.setOriginalValue(originalValue);
        List<SwiftTranslationErrorInfo> errorInfos = str.getErrorInfos();
        boolean existFlag = false;
        for (int i=0; i<errorInfos.size(); i++) {
            if (SwiftTransferUtil.compareStErrorInfo(errorInfo, errorInfos.get(i))) {
                existFlag = true;
                break;
            }
        }
        if (!existFlag) {
            str.addErrorInfo(errorInfo);
            if ("FAILURE".equalsIgnoreCase(errorInfo.getErrorType())) {
                throw new SwiftException(location + Mx2MtConstants.NEW_LINE + "-" + errorInfo.getErrorType() + ":" + errorInfo.getDescription());
            }
        }
    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        List<Mt2MxParseField> list = new ArrayList<>();
        list.add(new Camt054001Parse13DField());
        list.add(new Camt054001Parse20Field());
        list.add(new Camt054001Parse21Field());
        list.add(new Camt054001Parse25Field());
        list.add(new Camt054001Parse32AField());
        list.add(new Camt054001Parse50Field());
        list.add(new Camt054001Parse52For900Field());
        list.add(new Camt054001Parse52For910Field());
        list.add(new Camt054001Parse56Field());
        list.add(new Camt054001Parse72Field());
        return list;
    }
}
