package com.brilliance.swift.mt2mx.camt056001;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.ERROR;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.camt056001.impl.*;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.SwiftTransferUtil;
import com.brilliance.swift.vo.SwiftTranslationErrorInfo;
import com.brilliance.swift.vo.SwiftTranslationReport;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field79;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxCamt056001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "camt.056.001.08");
        Map<String, Object> appHdrMaps = (Map<String, Object>)jsonMaps.get("appHdr");
        appHdrMaps.put("msgDefIdr", "camt.056.001.08");
        AbstractMT abstractMT = context.get(AbstractMT.class);

        //初始化转换和不需要BLOCK4的转换
        Map<String, Object> fiToFIPmtCxlReqMaps = new HashMap<>();
        jsonMaps.put("fiToFIPmtCxlReq", fiToFIPmtCxlReqMaps);
        Map<String, Object> assgnmtMaps = new HashMap<>();
        fiToFIPmtCxlReqMaps.put("assgnmt", assgnmtMaps);
        assgnmtMaps.put("creDtTm", getCreateDateStr());

        Map<String, Object> undrlygMaps = new HashMap<>();
        JSONArray undrlygJsonArray = new JSONArray();
        undrlygJsonArray.add(undrlygMaps);
        fiToFIPmtCxlReqMaps.put("undrlyg", undrlygJsonArray);
        Map<String, Object> txInfMaps = new HashMap<>();
        JSONArray txInfJsonArray = new JSONArray();
        txInfJsonArray.add(txInfMaps);
        undrlygMaps.put("txInf",txInfJsonArray);
        String orgnlUETR = abstractMT.getSwiftMessage().getUETR();
        if (StringUtil.isNotEmpty(orgnlUETR)) {
            txInfMaps.put("orgnlUETR", orgnlUETR);
        }
        txInfMaps.put("orgnlEndToEndId",Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
        if (StringUtil.isNotEmpty(sendBic)) {
            Map<String, Object> assgnrMaps = new HashMap<>();
            assgnmtMaps.put("assgnr",assgnrMaps);
            Map<String, Object> agtMaps = new HashMap<>();
            assgnrMaps.put("agt",agtMaps);
            mt_to_mxBICFI(sendBic, agtMaps);
        }
        String receiverBic = (String) context.get(Mt2MxContextIdentifier.MT_RECEIVE_BIC, true);
        if (StringUtil.isNotEmpty(receiverBic)) {
            Map<String, Object> assgneMaps = new HashMap<>();
            assgnmtMaps.put("assgne",assgneMaps);
            Map<String, Object> agtMaps = new HashMap<>();
            assgneMaps.put("agt",agtMaps);
            mt_to_mxBICFI(receiverBic, agtMaps);
        }
        Map<String, Object> caseMaps = null;
        if(txInfMaps.containsKey("_case")){
            caseMaps = (Map<String, Object>) txInfMaps.get("_case");
        }else {
            caseMaps = new HashMap<>();
            txInfMaps.put("_case",caseMaps);
        }
        Map<String, Object> cretrMaps = new HashMap<>();
        Map<String, Object> agtMaps = new HashMap<>();
        Map<String, Object> finInstnIdMaps = new HashMap<>();
        Map<String, Object> pstlAdrMaps = new HashMap<>();
        caseMaps.put("cretr",cretrMaps);
        cretrMaps.put("agt",agtMaps);
        agtMaps.put("finInstnId",finInstnIdMaps);
        finInstnIdMaps.put("nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        finInstnIdMaps.put("pstlAdr",pstlAdrMaps);
        JSONArray adrLineJsonArray = new JSONArray();
        pstlAdrMaps.put("adrLine",adrLineJsonArray);
        adrLineJsonArray.add(Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
        Tag tag79 = abstractMT.getSwiftMessage().getBlock4().getTagByName("79");
        Field79 field79 = (Field79)tag79.asField();
        int index = field79.getValue().indexOf("/UETR/");
        String uetr = "";
        if(index>-1){
            uetr = field79.getValue().substring(index+6);
        }
        if(StringUtil.isEmpty(orgnlUETR)&&StringUtil.isEmpty(uetr)){
            buildSTErrorInfo(ERROR.T20087, "Block3/UETR", null);
        }
    }
    protected void buildSTErrorInfo(String errorCode, String location, String originalValue) {
        SwiftTranslationReport str = context.get(SwiftTranslationReport.class);
        SwiftTranslationErrorInfo errorInfo = SwiftTransferUtil.getSwiftTranslationErrorInfoByErrorCode(errorCode);
        errorInfo.setLocation(location);
        errorInfo.setOriginalValue(originalValue);
        List<SwiftTranslationErrorInfo> errorInfos = str.getErrorInfos();
        boolean existFlag = false;
        for (int i=0; i<errorInfos.size(); i++) {
            if (SwiftTransferUtil.compareStErrorInfo(errorInfo, errorInfos.get(i))) {
                existFlag = true;
                break;
            }
        }
        if (!existFlag) {
            str.addErrorInfo(errorInfo);
            if ("FAILURE".equalsIgnoreCase(errorInfo.getErrorType())) {
                throw new SwiftException(location + Mx2MtConstants.NEW_LINE + "-" + errorInfo.getErrorType() + ":" + errorInfo.getDescription());
            }
        }
    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        List<Mt2MxParseField> list = new ArrayList<>();
        list.add(new Camt056001Parse20Field());
        list.add(new Camt056001Parse21Field());
        list.add(new Camt056001Parse11SField());
        list.add(new Camt056001Parse79Field());
        list.add(new Camt056001Parse32AField());
        return list;
    }
}
