package com.brilliance.swift.mx2element;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.AbstractMX;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public abstract class AbstractMx2ElementCreator implements Mx2ElementCreator{

    protected AbstractMX abstractMX;

    protected String xmlStr;

    protected Map<String, Object> maps = new LinkedHashMap<>();

    public AbstractMX getAbstractMX() {
        return abstractMX;
    }

    public void setAbstractMX(AbstractMX abstractMX) {
        this.abstractMX = abstractMX;
    }

    public String getXmlStr() {
        return xmlStr;
    }

    public void setXmlStr(String xmlStr) {
        this.xmlStr = xmlStr;
    }

    private Object invokeHere(String methodName, Object where) {
        try {
            Method method = getClass().getMethod(methodName);
            return method.invoke(where);
        } catch (final NoSuchMethodException e) {
            throw new SwiftException("Method " + methodName + " does not exist in " + getClass());
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    public List<String> convertToLines(InputStream is) {
        BufferedReader bReader = new BufferedReader(new InputStreamReader(is));
        List<String> list = new ArrayList<>();
        String line = null;
        try {
            while((line = bReader.readLine())!=null){
                list.add(line);
            }
        } catch (IOException e) {
            throw new SwiftException(e.getMessage());
        } finally {
            try {
                bReader.close();
            } catch (IOException e) {
                throw new SwiftException(e.getMessage());
            }
        }
        return list;
    }

    @Override
    public Map<String, Object> buildElement() {
            //AbstractMX abstractMX = AbstractMX.parse(xmlStr);
            String messageType = abstractMX.getMxId().getBusinessProcess().name()
                                + abstractMX.getMxId().getFunctionality()
                                + abstractMX.getMxId().getVariant()
                                + abstractMX.getMxId().getVersion();
            maps.put("messageType", messageType);
            String gsonStr = abstractMX.toJson();
            Map<String, Object> mxGsonMaps = JSON.parseObject(gsonStr);
            String fileName = "/template/"+messageType+".properties";
            InputStream resourceAsStream = AbstractMx2ElementCreator.class.getResourceAsStream(fileName);
            if (resourceAsStream == null) {
                int versionLength = abstractMX.getMxId().getVersion().length();
                fileName = "/template/"+messageType.substring(0, messageType.length()-versionLength)+".properties";
                resourceAsStream = AbstractMx2ElementCreator.class.getResourceAsStream(fileName);

            }
            List<String> properties = StringUtil.inputStreamToLines(resourceAsStream);
            if (properties.size() > 0) {
                for (String property : properties) {
                    if (property.startsWith("#")) {
                        continue;//过滤注释
                    }
                    String[] strArr = property.split("=");
                    if (strArr.length != 2) {
                        continue;//过滤掉不符合规则的配置
                    }
                    String key = strArr[0];
                    String path = strArr[1];
                    Object value = getXmlValue(mxGsonMaps, path);
                    if (value != null) {
                        maps.put(key, value);
                    }
                }
            }

        return maps;
    }

    protected Object getXmlValue(Map<String, Object> jsonMaps, String path) {
        Object value = null;
        if (path.startsWith("@")) { //如果以@开始 表示取值规则采用的函数
            return invokeHere(path.substring(1), this);
        }
        if (path.indexOf("?") > 0) {
            String[] childPaths = path.split("\\?");
            for (String childPath : childPaths) {
                value = getXmlValue(jsonMaps, childPath);
                if (value != null) {
                    return value;
                }
            }
        }
        //获取数据类型，&分隔，默认是string
        String type = "string";
        if (path.indexOf("&") > -1) {
            type = path.substring(path.indexOf("&")+1);
            path = path.substring(0, path.indexOf("&"));
        }
        //需要用. split,然后循环遍历json获取值
        String[] paths = path.split("\\.");
        Map<String, Object> tmpMaps = jsonMaps;
        for (int i=0; i<paths.length; i++) {
            String p = paths[i];
            Object obj = tmpMaps.get(p);
            if (obj == null) return null;
            if (obj instanceof JSONArray) {
                JSONArray jsonArray = (JSONArray)obj;
                if (i == paths.length-1) {
                    return jsonArray;
                } else {
                    obj = jsonArray.get(0);
                    if (obj instanceof JSONObject) {
                        tmpMaps = (Map<String, Object>)obj; //不是最后一层
                    } else {
                        return jsonArray;
                    }
                }
            } else if (obj instanceof JSONObject) {
                tmpMaps = (Map<String, Object>)obj;
            } else {
                if ("bigdecimal".equals(type)) {
                    value = new BigDecimal(String.valueOf(obj));
                } else if ("boolean".equals(type)) {
                    value = Boolean.valueOf(String.valueOf(obj));
                } else {
                    value = String.valueOf(obj);
                }
                break;
            }
        }
        return value;
    }
}
