package com.brilliance.swift.mx2element;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2element.camt053001.Mx2ElementCamt053001Creator;
import com.brilliance.swift.mx2element.camt054001.Mx2ElementCamt054001Creator;
import com.brilliance.swift.mx2element.pacs008001.Mx2ElementPacs008001Creator;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.prowidesoftware.swift.model.mx.AbstractMX;

import java.util.LinkedHashMap;
import java.util.Map;

public class Mx2ElementCreatorManager {

    public String mx2Element(String xmlStr) throws SwiftException {
        AbstractMX abstractMX = AbstractMX.parse(xmlStr);
        String messageType = (abstractMX.getMxId().getBusinessProcess().name()
                + "."
                + abstractMX.getMxId().getFunctionality()
                + "."
                + abstractMX.getMxId().getVariant())
                //+ abstractMX.getMxId().getVersion())
                .trim();
        AbstractMx2ElementCreator creator = getCreator(messageType);
        creator.setAbstractMX(abstractMX);
        creator.setXmlStr(xmlStr);
        Map<String, Object> maps = creator.buildElement();
        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        return gson.toJson(maps, LinkedHashMap.class);
    }

    public AbstractMx2ElementCreator getCreator(String messageType) {
        if ("pacs.008.001".equals(messageType)) {
            return new Mx2ElementPacs008001Creator();
        } else if("camt.054.001".equals(messageType)){
            return new Mx2ElementCamt054001Creator();
        }else if("camt.053.001".equals(messageType)){
            return new Mx2ElementCamt053001Creator();
        }
        else {
            throw new SwiftException("Invalid message type");
        }
    }
}
