package com.brilliance.swift.mx2map;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.util.MessageUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.prowidesoftware.swift.model.mx.AbstractMX;
import org.dom4j.Document;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.LinkedHashMap;
import java.util.Map;

public abstract class AbstractMx2MapCreator implements Mx2MapCreator{

    protected String xmlStr;

    protected AbstractMX abstractMX;

    protected String localCode;

    protected Document document;

    protected Map<String, String> maps = new LinkedHashMap<>();

    public String getXmlStr() {
        return xmlStr;
    }

    public void setXmlStr(String xmlStr) {
        this.xmlStr = xmlStr;
    }

    public AbstractMX getAbstractMX() {
        return abstractMX;
    }

    public void setAbstractMX(AbstractMX abstractMX) {
        this.abstractMX = abstractMX;
    }

    public String getLocalCode() {
        return localCode;
    }

    public void setLocalCode(String localCode) {
        this.localCode = localCode;
    }

    public Document getDocument() {
        return document;
    }

    public void setDocument(Document document) {
        this.document = document;
    }

    protected String getPropertyValue(String key) {
        return MessageUtil.getPropertyValue(localCode, key);
    }

    protected void buildCommonHeader(String uetr) {
        String sendBic = abstractMX.getAppHdr().from();
        maps.put(getPropertyValue("app.header.sendBic"), sendBic);
        String receiverBic = abstractMX.getAppHdr().to();
        maps.put(getPropertyValue("app.header.receiverBic"), receiverBic);
        /*String bizMsgId = abstractMX.getAppHdr().reference();
        maps.put(getPropertyValue("app.header.bizMsgId"), bizMsgId);*/
        String msgDefId = abstractMX.getAppHdr().messageName();
        maps.put(getPropertyValue("app.header.msgDefId"), msgDefId);
        String bizSvc = abstractMX.getAppHdr().serviceName();
        maps.put(getPropertyValue("app.header.bizSvc"), bizSvc);
        XMLGregorianCalendar creDt = abstractMX.getAppHdr().creationDate();
        maps.put(getPropertyValue("app.header.creDt"), creDt.toXMLFormat());
        if (StringUtil.isNotEmpty(uetr)) {
            maps.put(getPropertyValue("uetr"), uetr);
        }
    }

    /**
     * 组装party信息
     * @param name 表示party的名字
     * @param path 表示party的xml路径
     */
    protected void buildPartyInfo(String name, String path) {
        int count = XmlUtil.getChildrenCount(document, path, null);
        if (count <= 0) return;
        maps.put(getPropertyValue(name), Mx2MtConstants.NEW_LINE+"\t");
        String partyName = XmlUtil.getXmlNodeValue(document, path + ".Nm");
        if (StringUtil.isNotEmpty(partyName)) {
            maps.put(getPropertyValue("name"), partyName);
        }
        String orgBicCode = XmlUtil.getXmlNodeValue(document, path + ".Id.OrgId.AnyBIC");
        if (StringUtil.isNotEmpty(orgBicCode)) {
            maps.put(getPropertyValue("organisation.bic"), orgBicCode);
        }
        String orgLei = XmlUtil.getXmlNodeValue(document, path + ".Id.OrgId.LEI");
        if (StringUtil.isNotEmpty(orgLei)) {
            maps.put(getPropertyValue("organisation.lei"), orgLei);
        }
        String orgOtherCode = XmlUtil.getXmlNodeValue(document, path + ".Id.OrgId.Othr.SchmeNm.Cd");
        if (StringUtil.isNotEmpty(orgOtherCode)) {
            maps.put(getPropertyValue("organisation.other.code"), orgOtherCode);
        }
        String orgOtherId = XmlUtil.getXmlNodeValue(document, path + ".Id.OrgId.Othr.Id");
        if (StringUtil.isNotEmpty(orgOtherId)) {
            maps.put(getPropertyValue("organisation.other.id"), orgOtherId);
        }
        String prvtBrithDate = XmlUtil.getXmlNodeValue(document, path + ".Id.PrvtId.DtAndPlcOfBirth.BirthDt");
        if (StringUtil.isNotEmpty(prvtBrithDate)) {
            maps.put(getPropertyValue("private.brith.date"), prvtBrithDate);
        }
        String prvtBrithCountry = XmlUtil.getXmlNodeValue(document, path + ".Id.PrvtId.DtAndPlcOfBirth.CtryOfBirth");
        if (StringUtil.isNotEmpty(prvtBrithCountry)) {
            maps.put(getPropertyValue("private.brith.country"), prvtBrithCountry);
        }
        String prvtBrithProvince = XmlUtil.getXmlNodeValue(document, path + ".Id.PrvtId.DtAndPlcOfBirth.PrvcOfBirth");
        if (StringUtil.isNotEmpty(prvtBrithProvince)) {
            maps.put(getPropertyValue("private.brith.province"), prvtBrithProvince);
        }
        String prvtBrithCity = XmlUtil.getXmlNodeValue(document, path + ".Id.PrvtId.DtAndPlcOfBirth.CityOfBirth");
        if (StringUtil.isNotEmpty(prvtBrithCity)) {
            maps.put(getPropertyValue("private.brith.city"), prvtBrithCity);
        }
        String prvtOtherCode = XmlUtil.getXmlNodeValue(document, path + ".Id/PrvtId/Othr/SchmeNm/Cd");
        if (StringUtil.isNotEmpty(prvtOtherCode)) {
            maps.put(getPropertyValue("private.other.code"), prvtOtherCode);
        }
        String prvtOtherId = XmlUtil.getXmlNodeValue(document, path + ".Id.PrvtId.Othr.Id");
        if (StringUtil.isNotEmpty(prvtOtherId)) {
            maps.put(getPropertyValue("private.other.id"), prvtOtherId);
        }
        String phoneNumber = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.PhneNb");
        if (StringUtil.isNotEmpty(phoneNumber)) {
            maps.put(getPropertyValue("phone.number"), phoneNumber);
        }
        String mobileNumber = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.MobNb");
        if (StringUtil.isNotEmpty(mobileNumber)) {
            maps.put(getPropertyValue("mobile.number"), mobileNumber);
        }
        String faxNumber = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.FaxNb");
        if (StringUtil.isNotEmpty(faxNumber)) {
            maps.put(getPropertyValue("fax.number"), faxNumber);
        }
        String emailAddress = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.EmailAdr");
        if (StringUtil.isNotEmpty(emailAddress)) {
            maps.put(getPropertyValue("email.number"), emailAddress);
        }
        String contactOtherType = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.Othr.ChanlTp");
        if (StringUtil.isNotEmpty(contactOtherType)) {
            maps.put(getPropertyValue("contact.other.type"), contactOtherType);
        }
        String contactOtherId = XmlUtil.getXmlNodeValue(document, path + ".CtctDtls.Othr.Id");
        if (StringUtil.isNotEmpty(contactOtherId)) {
            maps.put(getPropertyValue("contact.other.id"), contactOtherId);
        }
        buildAddressInfo(path+ ".PstlAdr");
    }

    /**
     * 组装address信息
     * @param path 表示address的xml路径
     */
    protected void buildAddressInfo(String path) {
        String addressType = XmlUtil.getXmlNodeValue(document, path + ".AdrTp.Cd");
        if (StringUtil.isNotEmpty(addressType)) {
            maps.put(getPropertyValue("address.type"), addressType);
        }
        String addressCountry = XmlUtil.getXmlNodeValue(document, path + ".Ctry");
        if (StringUtil.isNotEmpty(addressCountry)) {
            maps.put(getPropertyValue("address.country"), addressCountry);
        }
        String addressDepartment = XmlUtil.getXmlNodeValue(document, path + ".Dept");
        if (StringUtil.isNotEmpty(addressDepartment)) {
            maps.put(getPropertyValue("address.department"), addressDepartment);
        }
        String addressSubDepartment = XmlUtil.getXmlNodeValue(document, path + ".SubDept");
        if (StringUtil.isNotEmpty(addressSubDepartment)) {
            maps.put(getPropertyValue("address.sub.department"), addressSubDepartment);
        }
        String addressTownName = XmlUtil.getXmlNodeValue(document, path + ".TwnNm");
        if (StringUtil.isNotEmpty(addressTownName)) {
            maps.put(getPropertyValue("address.town.name"), addressTownName);
        }
        String addressStreetName = XmlUtil.getXmlNodeValue(document, path + ".StrtNm");
        if (StringUtil.isNotEmpty(addressStreetName)) {
            maps.put(getPropertyValue("address.street.name"), addressStreetName);
        }
        String addressBuildingName = XmlUtil.getXmlNodeValue(document, path + ".BldgNm");
        if (StringUtil.isNotEmpty(addressBuildingName)) {
            maps.put(getPropertyValue("address.building.name"), addressBuildingName);
        }
        String addressBuildingNumber = XmlUtil.getXmlNodeValue(document, path + ".BldgNb");
        if (StringUtil.isNotEmpty(addressBuildingNumber)) {
            maps.put(getPropertyValue("address.building.number"), addressBuildingNumber);
        }
        String addressFloor = XmlUtil.getXmlNodeValue(document, path + ".Flr");
        if (StringUtil.isNotEmpty(addressFloor)) {
            maps.put(getPropertyValue("address.floor"), addressFloor);
        }
        String addressRoom = XmlUtil.getXmlNodeValue(document, path + ".Room");
        if (StringUtil.isNotEmpty(addressRoom)) {
            maps.put(getPropertyValue("address.room"), addressRoom);
        }
        String addressPostCode = XmlUtil.getXmlNodeValue(document, path + ".PstCd");
        if (StringUtil.isNotEmpty(addressPostCode)) {
            maps.put(getPropertyValue("address.post.code"), addressPostCode);
        }
        String addressPostBox = XmlUtil.getXmlNodeValue(document, path + ".PstBx");
        if (StringUtil.isNotEmpty(addressPostBox)) {
            maps.put(getPropertyValue("address.post.box"), addressPostBox);
        }
        String addressLine = "";
        int addressLineCount = XmlUtil.getChildrenCount(document, path + ".AdrLine", null);
        if (addressLineCount > 0) {
            for (int i=0; i<addressLineCount; i++) {
                addressLine += XmlUtil.getXmlNodeValue(document, path + ".AdrLine("+i+")");
            }
        }
        if (StringUtil.isNotEmpty(addressLine)) {
            maps.put(getPropertyValue("address.line"), addressLine);
        }
    }
}
