package com.brilliance.swift.mx2map.camt052;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2map.AbstractMx2MapCreator;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.Map;

public class Mx2MapCamt052Creator extends AbstractMx2MapCreator {
    @Override
    public Map<String, String> mx2Map() throws SwiftException {
        try {
            Document document = DocumentHelper.parseText(xmlStr);
            Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
            String appHdrParentElementName = parentElementMaps.get("AppHdr");
            String grpHdrParentElementName = parentElementMaps.get("GrpHdr");
            String bodyParentElementName = parentElementMaps.get("Rpt");
            String sendBic = abstractMX.getAppHdr().from();
            maps.put(getPropertyValue("app.header.sendBic"), sendBic);
            String receiverBic = abstractMX.getAppHdr().to();
            maps.put(getPropertyValue("app.header.receiverBic"), receiverBic);
            String msgDefId = abstractMX.getAppHdr().messageName();
            maps.put(getPropertyValue("app.header.msgDefId"), msgDefId);
            String bizSvc = abstractMX.getAppHdr().serviceName();
            maps.put(getPropertyValue("app.header.bizSvc"), bizSvc);
            XMLGregorianCalendar creDt = abstractMX.getAppHdr().creationDate();
            maps.put(getPropertyValue("app.header.creDt"), creDt.toXMLFormat());
            String messageId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Id");
            if(StringUtil.isNotEmpty(messageId)){
                maps.put(Mx2MtConstants.NEW_LINE+getPropertyValue("message.id"), messageId);
            }
            String iban = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Acct.Id.IBAN");
            String acctId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Acct.Id.Othr.Id");
            String account = StringUtil.isNotEmpty(iban) ? iban : acctId;
            if (StringUtil.isNotEmpty(account)) {
                maps.put(getPropertyValue("statement.account"), account);
            }
            String elctrncSeqNb = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.LglSeqNb");
            if (StringUtil.isEmpty(elctrncSeqNb) || elctrncSeqNb.length() > 5) {
                elctrncSeqNb = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.ElctrncSeqNb");
            }
            if (StringUtil.isNotEmpty(elctrncSeqNb)) {
                maps.put(getPropertyValue("statement.sequence.number"), elctrncSeqNb);
            }
            String pageNumber = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.RptPgntn.PgNb");
            if (StringUtil.isNotEmpty(pageNumber)){
                maps.put(getPropertyValue("statement.page.number"), pageNumber);
            }
            String amount = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Acct.Ccy");
            if(StringUtil.isNotEmpty(amount)){
                maps.put(getPropertyValue("report.account.currency"),amount+"0");
            }
            String dateStr = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.CreDtTm");
            if(StringUtil.isEmpty(dateStr)){
                dateStr = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".GrpHdr.CreDtTm");
            }
            if(StringUtil.isEmpty(dateStr)){
                maps.put(getPropertyValue("report.creDtTm"),dateStr);
            }
            int entryCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Rpt.Ntry", null);
            if (entryCount > 0) {
                for (int i=0; i<entryCount; i++) {
                    String reversalIndicator = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").RvslInd");
                    String creditDebitIndicator = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").CdtDbtInd");
                    if (StringUtil.isNotEmpty(reversalIndicator) && Boolean.valueOf(reversalIndicator)) {
                        if (CdtDbtCode.DBIT.value().equals(creditDebitIndicator)) {
                            maps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("statement.entry.crdr")+"("+(i+1)+")", getPropertyValue("reversal.credit"));
                        } else {
                            maps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("statement.entry.crdr")+"("+(i+1)+")", getPropertyValue("reversal.debit"));
                        }
                    } else {
                        if (CdtDbtCode.DBIT.value().equals(creditDebitIndicator)) {
                            maps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("statement.entry.crdr")+"("+(i+1)+")", getPropertyValue("debit"));
                        } else {
                            maps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("statement.entry.crdr")+"("+(i+1)+")", getPropertyValue("credit"));
                        }
                    }
                    String valueDateStr = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").ValDt.Dt");
                    if (StringUtil.isNotEmpty(valueDateStr)) {
                        maps.put(getPropertyValue("statement.entry.valueDate")+"("+(i+1)+")", valueDateStr);
                    }
                    String amt = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").Amt");
                    String ccy = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").Amt@Ccy");
                    if (StringUtil.isNotEmpty(ccy)) {
                        maps.put(getPropertyValue("statement.entry.ccy")+"("+(i+1)+")", ccy);
                    }
                    if (StringUtil.isNotEmpty(amt)) {
                        maps.put(getPropertyValue("statement.entry.amt")+"("+(i+1)+")", amt);
                    }
                    String endToEndId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.Ntry("+i+").NtryDtls.TxDtls.Refs.EndToEndId");
                    if (StringUtil.isNotEmpty(endToEndId)) {
                        maps.put(getPropertyValue("statement.entry.endToEndId")+"("+(i+1)+")", endToEndId);
                    }
                    String sumc = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.TxsSummry.TtlCdtNtries.Sum");
                    String nbOfNtriesc = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.TxsSummry.TtlCdtNtries.NbOfNtries");
                    if(StringUtil.isNotEmpty(sumc)&&StringUtil.isNotEmpty(nbOfNtriesc)){
                        maps.put(Mx2MtConstants.NEW_LINE+getPropertyValue("report.transactionsSummary.totalCreditEntries.nbOfNtries"), nbOfNtriesc);
                        maps.put(getPropertyValue("report.transactionsSummary.totalCreditEntries.sum"), sumc);
                    }
                    String sumd = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.TxsSummry.TtlDbtNtries.Sum");
                    String nbOfNtriesd = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Rpt.TxsSummry.TtlDbtNtries.NbOfNtries");
                    if(StringUtil.isNotEmpty(sumd)&&StringUtil.isNotEmpty(nbOfNtriesd)){
                        maps.put(getPropertyValue("report.transactionsSummary.totalDebitEntries.nbOfNtries"), nbOfNtriesd);
                        maps.put(getPropertyValue("report.transactionsSummary.totalDebitEntries.sum"), sumd);
                    }
                }
            }

        } catch (DocumentException e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
        return maps;
    }
}
