package com.brilliance.swift.mx2map.camt057;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2map.AbstractMx2MapCreator;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.HashMap;
import java.util.Map;

public class Mx2MapCamt057Creator extends AbstractMx2MapCreator {
    @Override
    public Map<String, String> mx2Map() throws SwiftException {
        try {
            Document document = DocumentHelper.parseText(xmlStr);
            Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
            String appHdrParentElementName = parentElementMaps.get("AppHdr");
            String grpHdrParentElementName = parentElementMaps.get("GrpHdr");
            String bodyParentElementName = parentElementMaps.get("Ntfctn");
            String sendBic = abstractMX.getAppHdr().from();
            maps.put(getPropertyValue("app.header.sendBic"), sendBic);
            String receiverBic = abstractMX.getAppHdr().to();
            maps.put(getPropertyValue("app.header.receiverBic"), receiverBic);
            String msgDefId = abstractMX.getAppHdr().messageName();
            maps.put(getPropertyValue("app.header.msgDefId"), msgDefId);
            String bizSvc = abstractMX.getAppHdr().serviceName();
            maps.put(getPropertyValue("app.header.bizSvc"), bizSvc);
            XMLGregorianCalendar creDt = abstractMX.getAppHdr().creationDate();
            maps.put(getPropertyValue("app.header.creDt"), creDt.toXMLFormat());
            String messageId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".GrpHdr.MsgId");
            if(StringUtil.isNotEmpty(messageId)){
                maps.put(Mx2MtConstants.NEW_LINE+getPropertyValue("message.id"), messageId);
            }
            String accountId = "";
            String iban = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Acct.Id.IBAN");
            String id = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Acct.Id.Othr.Id");
            if(StringUtil.isNotEmpty(iban)){
                accountId = iban;
            }else if(StringUtil.isNotEmpty(id)){
                accountId = id;
            }else if(StringUtil.isEmpty(iban)&&StringUtil.isEmpty(id)) {
                String itemIban = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm(" + 0 + ").Acct.Id.IBAN");
                String itemId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm(" + 0 + ").Acct.Id.Othr.Id");
                if (StringUtil.isNotEmpty(itemIban)) {
                    accountId = itemIban;
                } else if (StringUtil.isNotEmpty(itemId)) {
                    accountId = itemId;
                }
            }
            if(StringUtil.isNotEmpty(accountId)){
                maps.put(getPropertyValue("notification.account.id"), accountId);
            }
            String valDtStr = "";
            String valDt = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.XpctdValDt");
            if(StringUtil.isNotEmpty(valDt)){
                valDtStr = valDt;
            }else {
                valDtStr = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").XpctdValDt");
            }
            if(StringUtil.isNotEmpty(valDtStr)){
                maps.put(getPropertyValue("notification.xpctdValDt"), valDtStr);
            }
            String endToEndId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").EndToEndId");
            if(StringUtil.isEmpty(endToEndId)){
                endToEndId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Id");
            }
            if(StringUtil.isNotEmpty(endToEndId)){
                maps.put(Mx2MtConstants.NEW_LINE+getPropertyValue("endToEnd.id"), endToEndId);
            }
            String ccy = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Amt@Ccy");
            if (StringUtil.isNotEmpty(ccy)) {
                maps.put(getPropertyValue("notification.amount.ccy"), ccy);
            }
            String amt = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Amt");
            if (StringUtil.isNotEmpty(amt)) {
                maps.put(getPropertyValue("notification.amount.amt"), amt);
            }
            String partyPath1 = ".Ntfctn.Dbtr.Pty";
            String partyPath2 = ".Ntfctn.Itm("+0+").Dbtr.Pty";
            String agentPath1 = ".Ntfctn.Dbtr.Agt";
            String agentPath2 = ".Ntfctn.Itm("+0+").Dbtr.Agt";
            Map<String,String> partyMessage1 = getPartyAndAgentAndAccountMessage(document,bodyParentElementName,partyPath1,"party");
            Map<String,String> partyMessage2 = getPartyAndAgentAndAccountMessage(document,bodyParentElementName,partyPath2,"party");
            Map<String,String> agentMessage1 = getPartyAndAgentAndAccountMessage(document,bodyParentElementName,agentPath1,"agent");
            Map<String,String> agentMessage2 = getPartyAndAgentAndAccountMessage(document,bodyParentElementName,agentPath2,"agent");
            maps.put(getPropertyValue("remitter.anyBIC"),partyMessage1.get("anyBIC"));
            maps.put(getPropertyValue("remitter.ctry"),partyMessage1.get("ctry"));
            maps.put(getPropertyValue("remitter.nm"),partyMessage1.get("nm"));
            maps.put(getPropertyValue("remitter.adrLine"),partyMessage1.get("adrLine"));
            maps.put(getPropertyValue("remitter.cd"),partyMessage1.get("cd"));
            maps.put(getPropertyValue("remitter.mmbId"),partyMessage1.get("mmbId"));
            /*String agentPath3 = ".Ntfctn.DbtrAgt";
            String agentPath4 = ".Ntfctn.Itm("+0+").DbtrAgt";
            maps.put("instructingBank",null);
            Map<String, String> agentMessage3 = getPartyAndAgentAndAccountMessage(document, bodyParentElementName, agentPath3);
            Map<String, String> agentMessage4 = getPartyAndAgentAndAccountMessage(document, bodyParentElementName, agentPath4);
            if(agentMessage3!=null||agentMessage4!=null){
                maps.clear();
            }
            if (agentMessage3!=null){
                maps.putAll(agentMessage3);
            }else if(agentMessage4!=null){
                maps.putAll(agentMessage4);
            }
            String agentPath5 = ".Ntfctn.IntrmyAgt";
            String agentPath6 = ".Ntfctn.Itm("+0+").IntrmyAgt";
            maps.put("intermediaryBank",null);
            Map<String, String> agentMessage5 = getPartyAndAgentAndAccountMessage(document, bodyParentElementName, agentPath5);
            Map<String, String> agentMessage6 = getPartyAndAgentAndAccountMessage(document, bodyParentElementName, agentPath6);
            if(agentMessage5!=null||agentMessage6!=null){
                maps.clear();
            }
            if(agentMessage5!=null){
                maps.putAll(agentMessage5);
            }else if(agentMessage6!=null){
                maps.putAll(agentMessage6);
            }*/
            /*int count1 = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Dbtr.Pty",null);
            int count2 = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Pty",null);
            int count3 = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Dbtr.Agt",null);
            int count4 = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt",null);*/
            /*String bicCode = "";
            String country = "";
            int addressLineCount = 0;
            String name = "";
            String clearingSystemCode = "";
            String ClearingSystemId = "";
            if(count1>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Pty.Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Pty.PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Dbtr.Pty.PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Pty.Nm");
            }
            if(count2>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Pty.Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Pty.PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Pty.PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Pty.Nm");
            }
            if(count3>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Agt.Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Agt.FinInstnId.PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Dbtr.Agt.FinInstnId.PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Dbtr.Agt.FinInstnId.Nm");
                clearingSystemCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + "..Ntfctn.Dbtr.Agt.FinInstnId.ClrSysMmbId.ClrSysId.Cd");
                ClearingSystemId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + "..Ntfctn.Dbtr.Agt.FinInstnId.ClrSysMmbId.MmbId");
            }
            if(count4>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + "..Ntfctn.Itm("+0+").Dbtr.Agt.Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.Nm");
                clearingSystemCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.ClrSysMmbId.ClrSysId.Cd");
                ClearingSystemId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.ClrSysMmbId.MmbId");
            }
            String address = "";
            for (int i = 0; i < addressLineCount; i++) {
                if(i!=1||i!=addressLineCount-1){
                    address = address + ",";
                }
                address = address + XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".Ntfctn.Itm("+0+").Dbtr.Agt.FinInstnId.PstlAdr.AdrLine("+i+")");
            }
            if(StringUtil.isNotEmpty(bicCode)){
                maps.put(getPropertyValue("notification.debtor.party.identification.organisationIdentification.anyBIC"), bicCode);
            }
            if(StringUtil.isNotEmpty(country)){
                maps.put(getPropertyValue("notification.debtor.party.postalAddress.ctry"), country);
            }
            if(StringUtil.isNotEmpty(address)){
                maps.put(getPropertyValue("notification.debtor.party.postalAddress.adrLine"), address);
            }
            if(StringUtil.isNotEmpty(name)){
                maps.put(getPropertyValue("notification.debtor.party.nm"), name);
            }
            if (StringUtil.isNotEmpty(clearingSystemCode)){
                maps.put(getPropertyValue("notification.debtor.agent.financialInstitutionIdentification.clearingSystemMemberIdentification.clearingSystemIdentification.cd"), clearingSystemCode);
            }
            if(StringUtil.isNotEmpty(ClearingSystemId)){
                maps.put(getPropertyValue("notification.debtor.agent.financialInstitutionIdentification.clearingSystemMemberIdentification.mmbId"), ClearingSystemId);
            }*/
        } catch (DocumentException e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
        return maps;
    }
    //choice选项为party和agent
    public Map<String, String> getPartyAndAgentAndAccountMessage(Document document, String bodyParentElementName, String partyAndAgentAndAccountPath,String choice){
        String bicCode = "";
        String country = "";
        int addressLineCount = 0;
        String address = "";
        String name = "";
        String clearingSystemCode = "";
        String ClearingSystemId = "";
        String account = "";
        Map<String, String> maps = new HashMap<>();
        if("party".equals(choice)){
            int countParty = XmlUtil.getChildrenCount(document, bodyParentElementName + partyAndAgentAndAccountPath,null);
            if(countParty>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath+".Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath+".PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + partyAndAgentAndAccountPath + ".PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".Nm");
                for (int i = 0; i < addressLineCount; i++) {
                    if(i!=1||i!=addressLineCount-1){
                        address = address + ",";
                    }
                    address = address + XmlUtil.getXmlNodeValue(document, bodyParentElementName + ".PstlAdr.AdrLine("+i+")");
                }
            }
        }
        if("agent".equals(choice)){
            int countAgent = XmlUtil.getChildrenCount(document, bodyParentElementName + partyAndAgentAndAccountPath,null);
            if(countAgent>0){
                bicCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".Id.OrgId.AnyBIC");
                country = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.PstlAdr.Ctry");
                addressLineCount = XmlUtil.getChildrenCount(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.PstlAdr.AdrLine",null);
                name = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.Nm");
                clearingSystemCode = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.ClrSysMmbId.ClrSysId.Cd");
                ClearingSystemId = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.ClrSysMmbId.MmbId");
                for (int i = 0; i < addressLineCount; i++) {
                    if(i!=1||i!=addressLineCount-1){
                        address = address + ",";
                    }
                    address = address + XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".FinInstnId.PstlAdr.AdrLine("+i+")");
                }
            }
        }
        if("account".equals(choice)){
            account = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".Id.IBAN");
            if(StringUtil.isEmpty(account)){
                account = XmlUtil.getXmlNodeValue(document, bodyParentElementName + partyAndAgentAndAccountPath + ".Id.Othr.Id");
            }
        }
        if(StringUtil.isNotEmpty(bicCode)){
            maps.put(getPropertyValue("anyBIC"), bicCode);
        }
        if(StringUtil.isNotEmpty(country)){
            maps.put(getPropertyValue("ctry"), country);
        }
        if(StringUtil.isNotEmpty(account)){
            maps.put(getPropertyValue("account"), account);
        }
        if(StringUtil.isNotEmpty(name)){
            maps.put(getPropertyValue("nm"), name);
        }
        if(StringUtil.isNotEmpty(address)){
            maps.put(getPropertyValue("adrLine"), address);
        }
        if (StringUtil.isNotEmpty(clearingSystemCode)){
            maps.put(getPropertyValue("cd"), clearingSystemCode);
        }
        if(StringUtil.isNotEmpty(ClearingSystemId)){
            maps.put(getPropertyValue("mmbId"), ClearingSystemId);
        }
        return maps;
    }
}
