package com.brilliance.swift.mx2map.pacs008;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2map.AbstractMx2MapCreator;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;
import java.util.Map;

public class Mx2MapPacs008Creator extends AbstractMx2MapCreator {
    @Override
    public Map<String, String> mx2Map() throws SwiftException {
        try {
            document = DocumentHelper.parseText(xmlStr);
            Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
            String appHdrParentElementName = parentElementMaps.get("AppHdr");
            String grpHdrParentElementName = parentElementMaps.get("GrpHdr");
            String cdtTrfTxInfParentElementName = parentElementMaps.get("CdtTrfTxInf");
            String uetr = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.PmtId.uetr");
            buildCommonHeader(uetr);
            String messageId = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.PmtId.InstrId");
            if (StringUtil.isEmpty(messageId)) {
                messageId = XmlUtil.getXmlNodeValue(document, grpHdrParentElementName + ".GrpHdr.MsgId");
            }
            if (StringUtil.isEmpty(messageId)) {
                messageId = abstractMX.getAppHdr().reference();
            }
            maps.put(getPropertyValue("message.id"), messageId);
            String endToEndId = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.PmtId.EndToEndId");
            maps.put(getPropertyValue("endToEnd.id"), endToEndId);
            String intrBkSttlmDtValue = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.IntrBkSttlmDt");
            if (StringUtil.isNotEmpty(intrBkSttlmDtValue)) {
                XMLGregorianCalendar intrBkSttlmDt = DatatypeFactory.newInstance().newXMLGregorianCalendar(intrBkSttlmDtValue);
                String intrBkSttlmDtStr = DateUtil.format(intrBkSttlmDt, "yyyy-MM-dd");
                maps.put(getPropertyValue("settlement.date"), intrBkSttlmDtStr);
            }
            String setlAmt = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.IntrBkSttlmAmt");
            String setlCcy = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.IntrBkSttlmAmt@Ccy");
            if (StringUtil.isNotEmpty(setlAmt) && StringUtil.isNotEmpty(setlCcy)) {
                maps.put(getPropertyValue("settlement.amt"), setlCcy + " " + NumberUtil.round(new BigDecimal(setlAmt), setlCcy));
            }
            String instdAmt = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.InstdAmt");
            String instdCcy = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.InstdAmt@Ccy");
            if (StringUtil.isNotEmpty(instdAmt) && StringUtil.isNotEmpty(instdCcy)) {
                maps.put(getPropertyValue("original.settlement.amt"), instdCcy + " " + NumberUtil.round(new BigDecimal(instdAmt), instdCcy));
            }
            String exchangeRateStr = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.XchgRate");
            if (StringUtil.isNotEmpty(exchangeRateStr)) {
                maps.put(getPropertyValue("exchange.rate"), exchangeRateStr);
            }
            String chrgBr = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgBr");
            if (StringUtil.isNotEmpty(chrgBr)) {
                maps.put(getPropertyValue("charge.by"), chrgBr);
            }
            int chrgsInfCount = XmlUtil.getChildrenCount(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgsInf", null);
            if (chrgsInfCount > 0) {
                String ccy = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgsInf(0).Amt@Ccy");
                String amt = XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgsInf(0).Amt");
                if (StringUtil.isNotEmpty(ccy) && StringUtil.isNotEmpty(amt)) {
                    BigDecimal sumAmt = new BigDecimal(amt);
                    if (chrgsInfCount > 1) {
                        for (int i=1; i<chrgsInfCount; i++) {
                            if (ccy.equals(XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgsInf("+i+").Amt@Ccy"))) {
                                BigDecimal tmpAmt = new BigDecimal(XmlUtil.getXmlNodeValue(document, cdtTrfTxInfParentElementName + ".CdtTrfTxInf.ChrgsInf("+i+").Amt"));
                                sumAmt = sumAmt.add(tmpAmt);
                            }
                        }
                    }
                    maps.put(getPropertyValue("charge.amt"), ccy + " " + NumberUtil.round(sumAmt, ccy));
                }
            }
            buildPartyInfo("debtor.info", cdtTrfTxInfParentElementName + ".CdtTrfTxInf.Dbtr");
            buildPartyInfo("creditor.info", cdtTrfTxInfParentElementName + ".CdtTrfTxInf.Cdtr");
        } catch (DocumentException e) {
            throw new SwiftException(e.getMessage());
        } catch (DatatypeConfigurationException e) {
            throw new SwiftException(e.getMessage());
        }
        return maps;
    }
}
