package com.brilliance.swift.mx2mt.mt103Retn.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtTagsGenerate;
import com.brilliance.swift.mx2mt.Mx2MtContextIdentifier;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;

import java.util.List;

/**
 *对于所有代理行，如果xml报文中有代理行的bic，并且账户和清算代码都存在，则MT账号取账户；
 *  * 如果代理行没有bic，MT账号优先取清算代码，没有的话再取账户。
 */
public class Field57AGenerate extends AbstractMx2MtTagsGenerate {

    private static String name_A = "57A";

    private static String name_D = "57D";

    @Override
    public void tagGenerate() throws SwiftException {
        context.set(Mx2MtContextIdentifier.MX_TO_MT_ERROR_LOCATION, "Block4/57a");
        SwiftMessage swiftMessage = context.get(SwiftMessage.class);
        List<Tag> tags = swiftMessage.getBlock4().getTags();
        String bicCode = mx_to_mtBICFI(bodyHdrParentElementName + ".TxInf.RtrChain.CdtrAgt");
        String mtClearSystemId = mx_to_mtClearingIdentifier(bodyHdrParentElementName + ".TxInf.RtrChain.CdtrAgt");
        String value = "";
        if (StringUtil.isNotEmpty(bicCode)) {
            if (StringUtil.isNotEmpty(mtClearSystemId)) {
                value = mtClearSystemId + Mx2MtConstants.NEW_LINE + bicCode;
            } else {
                value = bicCode;
            }
            tags.add(new Tag(name_A, value));
        } else {
            String name = getXmlNodeValue(bodyHdrParentElementName, document, "TxInf.RtrChain.CdtrAgt.FinInstnId.Nm");
            String countryCode = getXmlNodeValue(bodyHdrParentElementName, document, "TxInf.RtrChain.CdtrAgt.FinInstnId.PstlAdr.Ctry");
            int addressLineCount = getXmlNodeCounts(bodyHdrParentElementName, document, "TxInf.RtrChain.CdtrAgt.FinInstnId.PstlAdr.AdrLine");
            String mtNameAddress = "";
            if (addressLineCount > 0) {
                mtNameAddress = mx_to_mtFinancialInstitutionNameAndUnstructuredAddress(bodyHdrParentElementName+ ".TxInf.RtrChain.CdtrAgt.FinInstnId");
            } else if (StringUtil.isNotEmpty(countryCode)) {
                mtNameAddress = mx_to_mtFinancialInstitutionNameAndStructuredAddress(bodyHdrParentElementName+ ".TxInf.RtrChain.CdtrAgt.FinInstnId", false);
            } else if (StringUtil.isNotEmpty(name)) {
                mtNameAddress = mx_to_mtFinancialInstitutionNameAndUnstructuredAddress(bodyHdrParentElementName+ ".TxInf.RtrChain.CdtrAgt.FinInstnId");
            }
            //mapping转换文档 Translation Post Conditions -> POSTC004
            boolean exist56aFlag = exist56a(tags);
            if (StringUtil.isEmpty(mtNameAddress) && exist56aFlag) {
                mtNameAddress = Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
                if (StringUtil.isEmpty(mtClearSystemId)) {
                    mtClearSystemId = "/" + Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
                }
            }
            if (StringUtil.isEmpty(mtNameAddress)) return;//如果name和地址不存在，返回。
            if (StringUtil.isNotEmpty(mtClearSystemId)) {
                value = mtClearSystemId + Mx2MtConstants.NEW_LINE + mtNameAddress;
            } else {
                value = mtNameAddress;
            }
            value = mx_to_mtStartingLineCharacter(value, "Block4/:57D:");
            tags.add(new Tag(name_D, value));
        }
    }

    /**
     * 56a 是否存在
     * @param tags
     * @return
     */
    private boolean exist56a(List<Tag> tags) {
        boolean flag = false;
        if (tags != null && tags.size() > 0) {
            for (int i=0; i<tags.size(); i++) {
                Tag tag = tags.get(i);
                if ("56A".equals(tag.getName()) || "56D".equals(tag.getName())) {
                    flag = true;
                    break;
                }
            }
        }
        return flag;
    }
}
