package com.brilliance.swift.mx2mt.mt900910;

import com.brilliance.swift.constants.ERROR;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.mx2mt.AbstractMx2MtCreator;
import com.brilliance.swift.mx2mt.Mx2MtContextIdentifier;
import com.brilliance.swift.mx2mt.Mx2MtTagsGenerate;
import com.brilliance.swift.mx2mt.mt900910.impl.*;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.brilliance.swift.vo.common.EntryStatusCode;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import org.dom4j.Document;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class Mx2Mt900910Creator extends AbstractMx2MtCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        Document document = (Document)context.get(Mx2MtContextIdentifier.MX_XMl_DOCUMENT, true);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        context.set(Mx2MtContextIdentifier.APPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("AppHdr"));
        context.set(Mx2MtContextIdentifier.GRPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("GrpHdr"));
        context.set(Mx2MtContextIdentifier.BODY_PARENT_ELEMENT_NAME, parentElementMaps.get("Ntfctn"));
        String bodyParentPath = parentElementMaps.get("Ntfctn");
        //Translation Preconditions/PREC001
        int ntfctnCount = getXmlNodeCounts(bodyParentPath, document, "Ntfctn");
        int entryCount = getXmlNodeCounts(bodyParentPath, document, "Ntfctn.Ntry");
        int entryDetailsCount = getXmlNodeCounts(bodyParentPath, document, "Ntfctn.Ntry.NtryDtls");
        int txnDetailsCount = getXmlNodeCounts(bodyParentPath, document, "Ntfctn.Ntry.NtryDtls.TxDtls");
        if (ntfctnCount > 1 || entryCount > 1 || entryDetailsCount > 1 || txnDetailsCount > 1) {
            buildSTErrorInfo(ERROR.T20053, "TranslationPrecondition",null);
        }
        //Translation Preconditions/PREC002
        String status = getXmlNodeValue(bodyParentPath, document, "Ntfctn.Ntry.Sts.Cd");
        if (!EntryStatusCode.BOOK.value().equals(status)) {
            buildSTErrorInfo(ERROR.T20038, "TranslationPrecondition",null);
        }
        //Translation Preconditions/PREC005
        String valueDateStr = getXmlNodeValue(bodyParentPath, document, "Ntfctn.Ntry.ValDt.Dt");
        String setlDateStr = getXmlNodeValue(bodyParentPath, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdDts.IntrBkSttlmDt");
        if (StringUtil.isEmpty(valueDateStr) && StringUtil.isEmpty(setlDateStr)) {
            buildSTErrorInfo(ERROR.T20078, "TranslationPrecondition",null);
        }
    }

    @Override
    public List<Mx2MtTagsGenerate> getGenerateTagList() {
        List<Mx2MtTagsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field21Generate());
        fieldsGenerateList.add(new Field25aGenerate());
        fieldsGenerateList.add(new Field13DGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field50aGenerate());
        fieldsGenerateList.add(new Field52aFor900Generate());
        fieldsGenerateList.add(new Field52aFor910Generate());
        fieldsGenerateList.add(new Field56aGenerate());
        fieldsGenerateList.add(new Field72Generate());
        return fieldsGenerateList;
    }

    @Override
    protected String getMtType() {
        String mtType = "";
        Document document = (Document)context.get(Mx2MtContextIdentifier.MX_XMl_DOCUMENT);
        String bodyParentPath = (String)context.get(Mx2MtContextIdentifier.BODY_PARENT_ELEMENT_NAME, true);
        String cdtDbtInd = getXmlNodeValue(bodyParentPath, document, "Ntfctn.Ntry.CdtDbtInd");
        if (CreditDebitCode.DBIT.value().equals(cdtDbtInd)) {
            mtType = Mx2MtConstants.MT_TYPE_900;
        } else if (CreditDebitCode.CRDT.value().equals(cdtDbtInd)) {
            mtType = Mx2MtConstants.MT_TYPE_910;
        }
        return mtType;
    }
}
