package com.brilliance.swift.mx2mt.mt941942.impl;

import com.brilliance.swift.constants.ERROR;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtTagsGenerate;
import com.brilliance.swift.mx2mt.Mx2MtContextIdentifier;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;
import java.util.List;

/**

 */
public class Field61Generate extends AbstractMx2MtTagsGenerate {

    private static String name_61 = "61";

    private static String name_86 = "86";

    @Override
    public void tagGenerate() throws SwiftException {
        try {
            String mtType = (String) context.get(Mx2MtContextIdentifier.MX_TO_MT_TYPE);
            if(Mx2MtConstants.MT_TYPE_941.equals(mtType)){
                return;
            }
            SwiftMessage swiftMessage = context.get(SwiftMessage.class);
            List<Tag> tags = swiftMessage.getBlock4().getTags();
            int entryCount = getXmlNodeCounts(bodyHdrParentElementName, document, "Rpt.Ntry");
            if(entryCount>190){
                buildSTErrorInfo(ERROR.T20110, "BLOCK4/:61:",entryCount+"");
                return;
            }
            String ccy2 = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Acct.Ccy");
            if (entryCount > 0) {
                for (int i = 0; i < entryCount; i++) {
                    String value = "";
                    String valueDateStr = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").ValDt.Dt");
                    if (StringUtil.isNotEmpty(valueDateStr)) {
                        XMLGregorianCalendar valueDate = DatatypeFactory.newInstance().newXMLGregorianCalendar(valueDateStr);
                        value += DateUtil.format(valueDate, "yyMMdd");
                    }
                    String bookDateStr = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").BookgDt.Dt");
                    if (StringUtil.isNotEmpty(bookDateStr)) {
                        XMLGregorianCalendar bookDate = DatatypeFactory.newInstance().newXMLGregorianCalendar(bookDateStr);
                        value += DateUtil.format(bookDate, "MMdd");
                    }
                    String code = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").Sts.Cd");
                    String reversalIndicator = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").RvslInd");
                    String creditDebitIndicator = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").CdtDbtInd");
                    if("BOOK".equals(code)){
                        if (StringUtil.isNotEmpty(reversalIndicator) && Boolean.valueOf(reversalIndicator)) {
                            if (CdtDbtCode.DBIT.value().equals(creditDebitIndicator)) {
                                value += "RC";
                            } else {
                                value += "RD";
                            }
                        } else {
                            if (CdtDbtCode.DBIT.value().equals(creditDebitIndicator)) {
                                value += "D";
                            } else {
                                value += "C";
                            }
                        }
                    }else if("FUTR".equals(code)||"PDNG".equals(code)||"INFO".equals(code)){
                        if (CdtDbtCode.DBIT.value().equals(creditDebitIndicator)) {
                            value += "ED";
                        } else {
                            value += "EC";
                        }
                    }
                    String amt = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").Amt");
                    String ccy = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").Amt@Ccy");
                    if (StringUtil.isNotEmpty(amt) && StringUtil.isNotEmpty(ccy)) {
                        String amount = NumberUtil.formatAmt(new BigDecimal(amt), ccy);
                        if (StringUtil.isNotEmpty(ccy2) && !ccy2.equals(ccy)) {
                            buildSTErrorInfo(ERROR.T20116, "Block4/61/"+(i+1), null);
                        }
                        if (NumberUtil.judgeAmtStrExceedLength(amt, 14)) {
                            buildSTErrorInfo(ERROR.T20113, "Block4/61/"+(i+1), amt);
                        }
                        value += amount;
                    }
                    value += "NTRF";
                    String endToEndId = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry(" + i + ").NtryDtls.TxDtls.Refs.EndToEndId");
                    if (StringUtil.isNotEmpty(endToEndId)) {
                        if (endToEndId.length() > 16) {
                            buildSTErrorInfo(ERROR.T0000T, "BLOCK4/:61:", endToEndId);
                            endToEndId = endToEndId.substring(0, 15) + "+";
                        }
                        value += endToEndId;
                    } else {
                        value += Mx2MtConstants.MT_21_DEFAULT_VALUE;
                    }
                    String accountServicerReference = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry("+i+").AcctSvcrRef");
                    if (StringUtil.isNotEmpty(accountServicerReference)) {
                        if (accountServicerReference.length() > 16) {
                            buildSTErrorInfo(ERROR.T0000T, "BLOCK4/:61:", accountServicerReference);
                            accountServicerReference = accountServicerReference.substring(0, 15) + "+";
                        }
                        value += "//" + accountServicerReference;
                    }
                    String additionalInformation = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry("+i+").NtryDtls.TxDtls.AddtlTxInf");
                    if (StringUtil.isNotEmpty(additionalInformation)) {
                        if (additionalInformation.length() > 34) {
                            buildSTErrorInfo(ERROR.T0000T, "BLOCK4/:61:", additionalInformation);
                            additionalInformation = additionalInformation.substring(0, 33) + "+";
                        }
                        value += Mx2MtConstants.NEW_LINE + additionalInformation;
                    }
                    tags.add(new Tag(name_61, value));
                    String additionalEntryInformation = getXmlNodeValue(bodyHdrParentElementName, document, "Rpt.Ntry("+i+").AddtlNtryInf");
                    if (StringUtil.isNotEmpty(additionalEntryInformation)) {
                        if (additionalEntryInformation.length() > 390) {
                            buildSTErrorInfo(ERROR.T0000T, "BLOCK4/:61:", additionalEntryInformation);
                            additionalEntryInformation = additionalEntryInformation.substring(0, 389) + "+";
                        }
                        String value86 = StringUtil.getStringByEnter(additionalEntryInformation, 65, 6);
                        tags.add(new Tag(name_86, value86));
                    }
                }
            }
        } catch (DatatypeConfigurationException e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
